from typing import Optional

from norman_objects.shared.queries.query_constraints import QueryConstraints
from norman_objects.shared.security.sensitive import Sensitive
from norman_objects.shared.status_flags.status_flag import StatusFlag
from pydantic import TypeAdapter

from norman_core.clients.http_client import HttpClient


class ModelFlags:
    @staticmethod
    async def get_model_status_flags(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json = constraints.model_dump(mode="json")

        response = await http_client.post("persist/model/flags/get", token, json=json)
        return TypeAdapter(dict[str, list[StatusFlag]]).validate_python(response)

    @staticmethod
    async def get_asset_status_flags(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json = constraints.model_dump(mode="json")

        response = await http_client.post("persist/asset/flags/get", token, json=json)
        return TypeAdapter(dict[str, list[StatusFlag]]).validate_python(response)



from typing import Optional

from norman_objects.shared.models.model_base import ModelBase
from norman_objects.shared.requests.get_models_request import GetModelsRequest
from norman_objects.shared.security.sensitive import Sensitive
from pydantic import TypeAdapter

from norman_core.clients.http_client import HttpClient


class ModelBases:
    @staticmethod
    async def get_model_bases(http_client: HttpClient, token: Sensitive[str], request: Optional[GetModelsRequest] = None):
        if request is None:
            request = GetModelsRequest(constraints=None, finished_models=True)
        json = request.model_dump(mode="json")

        response = await http_client.post("persist/models/bases/get", token, json=json)
        return TypeAdapter(dict[str, ModelBase]).validate_python(response)



class _HttpConfig:
    base_url = "https://api.<environment-name>.<sandbox-name>.public.norman-ai.com/v0"
    timeout_seconds = 10

class _IOConfig:
    chunk_size = 2 ** 16
    flush_size = 8 * (1024 ** 2)

class AppConfig:
    http = _HttpConfig
    io = _IOConfig



import asyncio
import base64
import contextlib
from typing import AsyncGenerator

from cryptography.hazmat.primitives.ciphers import Cipher, algorithms
from norman_objects.services.file_push.pairing.socket_pairing_response import SocketPairingResponse
from norman_utils_external.streaming_utils import StreamingUtils, AsyncBufferedReader
from xxhash import xxh3_64

from norman_core._app_config import AppConfig


class SocketClient:
    @staticmethod
    async def write_and_digest(socket_info: SocketPairingResponse, asset_stream: AsyncBufferedReader):
        hash_stream = xxh3_64()
        body_stream = StreamingUtils.process_read_stream(asset_stream, hash_stream.update, AppConfig.io.chunk_size, False)

        async for _ in SocketClient.write(socket_info, body_stream):
            ...

        return hash_stream.hexdigest()

    @staticmethod
    async def write(socket_info: SocketPairingResponse, file_stream: AsyncGenerator[bytes, None]):
        authentication_header = base64.b64decode(socket_info.authentication_header)
        encryptor = SocketClient._create_encryptor(socket_info)

        body_stream = StreamingUtils.chain_streams([authentication_header], file_stream)

        stream_reader, stream_writer = await asyncio.open_connection(socket_info.host, socket_info.port)
        try:
            async for chunk in body_stream:
                encrypted = encryptor.update(chunk)
                stream_writer.write(encrypted)
                if stream_writer.transport.get_write_buffer_size() >= AppConfig.io.flush_size:
                    await stream_writer.drain()
                await stream_writer.drain()
                yield chunk
        finally:
            stream_writer.close()
            with contextlib.suppress(ConnectionResetError):
                await stream_writer.wait_closed()

    @staticmethod
    def _create_encryptor(pairing_response: SocketPairingResponse):
        key_bytes = base64.b64decode(pairing_response.encryption_key)
        base_nonce12 = base64.b64decode(pairing_response.nonce)

        counter_nonce4 = (0).to_bytes(4, "little")
        full_nonce16 = counter_nonce4 + base_nonce12

        cipher = Cipher(algorithms.ChaCha20(key_bytes, full_nonce16), mode=None)
        encryptor = cipher.encryptor()

        return encryptor



from norman_objects.shared.security.sensitive import Sensitive

from norman_core.clients.http_client import HttpClient, ResponseEncoding


class Retrieve:
    @staticmethod
    async def get_model_asset(http_client: HttpClient, token: Sensitive[str], account_id: str, model_id: str, asset_id: str):
        endpoint = f"retrieve/asset/{account_id}/{model_id}/{asset_id}"
        return await http_client.get(endpoint, token, response_encoding=ResponseEncoding.Iterator)

    @staticmethod
    async def get_invocation_input(http_client: HttpClient, token: Sensitive[str], account_id: str, model_id: str, invocation_id: str, input_id: str):
        endpoint = f"retrieve/input/{account_id}/{model_id}/{invocation_id}/{input_id}"
        return await http_client.get(endpoint, token, response_encoding=ResponseEncoding.Iterator)

    @staticmethod
    async def get_invocation_output(http_client: HttpClient, token: Sensitive[str], account_id: str, model_id: str, invocation_id: str, output_id: str):
        endpoint = f"retrieve/output/{account_id}/{model_id}/{invocation_id}/{output_id}"
        return await http_client.get(endpoint, token, response_encoding=ResponseEncoding.Iterator)


from norman_objects.services.authenticate.register.register_auth_factor_request import RegisterAuthFactorRequest
from norman_objects.services.authenticate.register.register_email_request import RegisterEmailRequest
from norman_objects.services.authenticate.register.register_password_request import RegisterPasswordRequest
from norman_objects.services.authenticate.register.resend_email_verification_code_request import ResendEmailVerificationCodeRequest
from norman_objects.shared.authentication.account_authentication_methods import AccountAuthenticationMethods
from norman_objects.shared.security.sensitive import Sensitive

from norman_core.clients.http_client import HttpClient


class Register:
    @staticmethod
    async def get_authentication_factors(http_client: HttpClient, token: Sensitive[str], account_id: str):
        response = await http_client.get(f"authenticate/register/get/authentication/factors/{account_id}", token)
        return AccountAuthenticationMethods.model_validate(response)

    @staticmethod
    async def generate_api_key(http_client: HttpClient, token: Sensitive[str], register_key_request: RegisterAuthFactorRequest):
        api_key = await http_client.post("authenticate/generate/key", token, json=register_key_request.model_dump(mode="json"))
        return api_key

    @staticmethod
    async def register_password(http_client: HttpClient, token: Sensitive[str], register_password_request: RegisterPasswordRequest):
        await http_client.post("authenticate/register/password", token, json=register_password_request.model_dump(mode="json"))

    @staticmethod
    async def register_email(http_client: HttpClient, token: Sensitive[str], register_email_request: RegisterEmailRequest):
        await http_client.post("authenticate/register/email", token, json=register_email_request.model_dump(mode="json"))


    @staticmethod
    async def verify_email(http_client: HttpClient, token: Sensitive[str], email: str, code: str):
        await http_client.post(f"authenticate/register/email/verify/{email}/{code}", token)

    @staticmethod
    async def resend_email_otp(http_client: HttpClient, token: Sensitive[str], resend_email_verification_code_request: ResendEmailVerificationCodeRequest):
        await http_client.post("authenticate/register/email/resend/otp", token, json=resend_email_verification_code_request.model_dump(mode="json"))



from typing import Optional

from norman_objects.shared.models.model import Model
from norman_objects.shared.models.model_preview import ModelPreview
from norman_objects.shared.queries.query_constraints import QueryConstraints
from norman_objects.shared.requests.get_models_request import GetModelsRequest
from norman_objects.shared.security.sensitive import Sensitive
from pydantic import TypeAdapter

from norman_core.clients.http_client import HttpClient


class Models:
    @staticmethod
    async def get_models(http_client: HttpClient, token: Sensitive[str], request: Optional[GetModelsRequest] = None):
        if request is None:
            request = GetModelsRequest(constraints=None, finished_models=True)
        json = request.model_dump(mode="json")

        response = await http_client.post("persist/models/get", token, json=json)
        return TypeAdapter(dict[str, Model]).validate_python(response)

    @staticmethod
    async def create_models(http_client: HttpClient, token: Sensitive[str], models: list[Model]):
        json = None
        if models is not None:
            json = TypeAdapter(list[Model]).dump_python(models, mode="json")

        response = await http_client.post("persist/models/", token, json=json)
        return TypeAdapter(dict[str, Model]).validate_python(response)

    @staticmethod
    async def upgrade_models(http_client: HttpClient, token: Sensitive[str], models: list[Model]):
        json = None
        if models is not None:
            json = TypeAdapter(list[Model]).dump_python(models, mode="json")

        response = await http_client.post("persist/models/version", token, json=json)
        return TypeAdapter(dict[str, Model]).validate_python(response)

    @staticmethod
    async def replace_models(http_client: HttpClient, token: Sensitive[str], models: list[Model]):
        json = None
        if models is not None:
            json = TypeAdapter(list[Model]).dump_python(models, mode="json")

        response = await http_client.patch("persist/models/", token, json=json)
        return TypeAdapter(dict[str, Model]).validate_python(response)

    @staticmethod
    async def set_active_model(http_client: HttpClient, token: Sensitive[str], model_previews: list[ModelPreview]):
        json = None
        if model_previews is not None:
            json = TypeAdapter(list[ModelPreview]).dump_python(model_previews, mode="json")

        response = await http_client.patch("persist/models/version", token, json=json)
        return TypeAdapter(list[ModelPreview]).validate_python(response)

    @staticmethod
    async def delete_models(http_client: HttpClient, token: Sensitive[str], constraints: QueryConstraints):
        json = constraints.model_dump()

        affected_entities_count: int = await http_client.delete("persist/models/", token, json=json)
        return affected_entities_count



from typing import Optional

from norman_objects.shared.queries.query_constraints import QueryConstraints
from norman_objects.shared.security.sensitive import Sensitive
from norman_objects.shared.status_flags.status_flag import StatusFlag
from pydantic import TypeAdapter

from norman_core.clients.http_client import HttpClient


class InvocationFlags:
    @staticmethod
    async def get_invocation_status_flags(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json=constraints.model_dump(mode="json")

        response = await http_client.post("/persist/invocation/flags/get", token, json=json)
        return TypeAdapter(dict[str, list[StatusFlag]]).validate_python(response)

    @staticmethod
    async def get_input_status_flags(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json=constraints.model_dump(mode="json")

        response = await http_client.post("/persist/input/flags/get", token, json=json)
        return TypeAdapter(dict[str, list[StatusFlag]]).validate_python(response)

    @staticmethod
    async def get_output_status_flags(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json=constraints.model_dump(mode="json")

        response = await http_client.post("/persist/output/flags/get", token, json=json)
        return TypeAdapter(dict[str, list[StatusFlag]]).validate_python(response)



from norman_objects.services.file_push.checksum.checksum_request import ChecksumRequest
from norman_objects.services.file_push.pairing.socket_asset_pairing_request import SocketAssetPairingRequest
from norman_objects.services.file_push.pairing.socket_input_pairing_request import SocketInputPairingRequest
from norman_objects.services.file_push.pairing.socket_pairing_response import SocketPairingResponse
from norman_objects.shared.security.sensitive import Sensitive

from norman_core.clients.http_client import HttpClient


class FilePush:
    @staticmethod
    async def allocate_socket_for_asset(http_client: HttpClient, token: Sensitive[str], pairing_request: SocketAssetPairingRequest):
        response = await http_client.post("file-push/socket/pair/asset", token, json=pairing_request.model_dump(mode="json"))
        return SocketPairingResponse.model_validate(response)

    @staticmethod
    async def allocate_socket_for_input(http_client: HttpClient, token: Sensitive[str], pairing_request: SocketInputPairingRequest):
        response = await http_client.post("file-push/socket/pair/input", token, json=pairing_request.model_dump(mode="json"))
        return SocketPairingResponse.model_validate(response)

    @staticmethod
    async def complete_file_transfer(http_client: HttpClient, token: Sensitive[str], checksum_request: ChecksumRequest):
        await http_client.post("file-push/socket/complete", token, json=checksum_request.model_dump(mode="json"))



from norman_core.services.persist.accounts import Accounts
from norman_core.services.persist.invocation_flags import InvocationFlags
from norman_core.services.persist.invocations import Invocations
from norman_core.services.persist.model_bases import ModelBases
from norman_core.services.persist.model_flags import ModelFlags
from norman_core.services.persist.models import Models
from norman_core.services.persist.notifications import Notifications


class Persist:
    accounts = Accounts
    invocation_flags = InvocationFlags
    invocations = Invocations
    model_bases = ModelBases
    model_flags = ModelFlags
    models = Models
    notifications = Notifications


__all__ = ["Persist"]



from norman_objects.services.authenticate.login.login_response import LoginResponse
from norman_objects.services.authenticate.signup.signup_email_request import SignupEmailRequest
from norman_objects.services.authenticate.signup.signup_password_request import SignupPasswordRequest
from norman_objects.shared.accounts.account import Account

from norman_core.clients.http_client import HttpClient


class Signup:
    @staticmethod
    async def signup_default(http_client: HttpClient):
        response = await http_client.put("authenticate/signup/default")
        return LoginResponse.model_validate(response)

    @staticmethod
    async def signup_with_password(http_client: HttpClient, signup_request: SignupPasswordRequest):
        response = await http_client.put("authenticate/signup/password", json=signup_request.model_dump(mode="json"))
        return Account.model_validate(response)

    @staticmethod
    async def signup_with_email(http_client: HttpClient, signup_request: SignupEmailRequest):
        response = await http_client.put("authenticate/signup/email", json=signup_request.model_dump(mode="json"))
        return Account.model_validate(response)



from typing import Optional

from norman_objects.shared.invocations.invocation import Invocation
from norman_objects.shared.queries.query_constraints import QueryConstraints
from norman_objects.shared.security.sensitive import Sensitive
from pydantic import TypeAdapter

from norman_core.clients.http_client import HttpClient


class Invocations:
    @staticmethod
    async def get_invocations(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json = constraints.model_dump(mode="json")

        response = await http_client.post("persist/invocations/get", token, json=json)
        return TypeAdapter(dict[str, Invocation]).validate_python(response)

    @staticmethod
    async def create_invocations(http_client: HttpClient, token: Sensitive[str], invocations: list[Invocation]):
        json = TypeAdapter(list[Invocation]).dump_python(invocations, mode="json")

        response = await http_client.post("persist/invocations", token, json=json)
        return TypeAdapter(list[Invocation]).validate_python(response)

    @staticmethod
    async def create_invocations_by_model_names(http_client: HttpClient, token: Sensitive[str], model_name_counter: dict[str, int]):
        response = await http_client.post("persist/invocations/by-name", token, json=model_name_counter)
        return TypeAdapter(list[Invocation]).validate_python(response)

    @staticmethod
    async def get_invocation_history(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json = constraints.model_dump(mode="json")

        response = await http_client.post("persist/invocation/history/get", token, json=json)
        return TypeAdapter(dict[str, Invocation]).validate_python(response)



from typing import Optional

import httpx
from httpx import Response
from norman_objects.shared.security.sensitive import Sensitive
from typing_extensions import Unpack

from norman_core._app_config import AppConfig
from norman_core.clients.objects.request_kwargs import RequestKwargs
from norman_core.clients.objects.response_encoding import ResponseEncoding


class HttpClient:
    def __init__(self, base_url: Optional[str] = None, timeout: Optional[float] = None):
        if base_url is None:
            base_url = AppConfig.http.base_url
        if timeout is None:
            timeout = AppConfig.http.timeout_seconds

        self._headers = {
            "Content-Type": "application/json"
        }

        self._client = httpx.AsyncClient(
            base_url=base_url,
            timeout=timeout
        )

    async def request(self, method: str, endpoint: str, token: Optional[Sensitive[str]] = None, *, response_encoding = ResponseEncoding.Json, **kwargs: Unpack[RequestKwargs]):
        headers = self._create_headers(token)
        request = self._client.build_request(method, endpoint, headers=headers, **kwargs)

        stream_response = response_encoding == ResponseEncoding.Iterator
        response = await self._client.send(request, stream=stream_response)
        return self._parse_response(response, response_encoding)

    async def get(self, endpoint: str, token: Optional[Sensitive[str]] = None, *, response_encoding = ResponseEncoding.Json, **kwargs: Unpack[RequestKwargs]):
        return await self.request("GET", endpoint, token, response_encoding=response_encoding, **kwargs)

    async def post(self, endpoint: str, token: Optional[Sensitive[str]] = None, *, response_encoding = ResponseEncoding.Json, **kwargs: Unpack[RequestKwargs]):
        return await self.request("POST", endpoint, token, response_encoding=response_encoding, **kwargs)

    async def put(self, endpoint: str, token: Optional[Sensitive[str]] = None, *, response_encoding = ResponseEncoding.Json, **kwargs: Unpack[RequestKwargs]):
        return await self.request("PUT", endpoint, token, response_encoding=response_encoding, **kwargs)

    async def patch(self, endpoint: str, token: Optional[Sensitive[str]] = None, *, response_encoding = ResponseEncoding.Json, **kwargs: Unpack[RequestKwargs]):
        return await self.request("PATCH", endpoint, token, response_encoding=response_encoding, **kwargs)

    async def delete(self, endpoint: str, token: Optional[Sensitive[str]] = None, *, response_encoding = ResponseEncoding.Json, **kwargs: Unpack[RequestKwargs]):
        return await self.request("DELETE", endpoint, token, response_encoding=response_encoding, **kwargs)

    async def post_multipart(self, endpoint: str, token: Sensitive[str], *, response_encoding = ResponseEncoding.Json, **kwargs: Unpack[RequestKwargs]):
        headers = {
            "Authorization": f"Bearer {token.value()}",
        }

        data = kwargs.get("data", {})
        files = kwargs.get("files", {})

        response = await self._client.request(
            "POST",
            endpoint,
            headers=headers,
            data=data,
            files=files
        )
        return self._parse_response(response, response_encoding)

    def _create_headers(self, token: Optional[Sensitive[str]]):
        headers = self._headers.copy()
        if token is not None:
            headers["Authorization"] = f"Bearer {token.value()}"
        return headers

    @staticmethod
    def _parse_response(response: httpx.Response, response_encoding: ResponseEncoding):
        response.raise_for_status()

        if response_encoding == ResponseEncoding.Bytes:
            return response.content
        if response_encoding == ResponseEncoding.Iterator:
            return response.headers, HttpClient._response_iterator(response)
        if response_encoding == ResponseEncoding.Json:
            return response.json()
        if response_encoding == ResponseEncoding.Text:
            return response.text
        raise ValueError(f"Invalid response encoding: {response_encoding}")

    @staticmethod
    async def _response_iterator(response: Response):
        try:
            async for chunk in response.aiter_bytes():
                if chunk is not None and len(chunk) > 0:
                    yield chunk
        finally:
            await response.aclose()

    async def close(self):
        await self._client.aclose()

    async def __aenter__(self):
        await self._client.__aenter__()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self._client.__aexit__(exc_type, exc, tb)
        return self

    def __repr__(self):
        return f"<Norman.ApiClient base_url={self._client.base_url} closed={self._client.is_closed}>"


from typing import Optional

from norman_objects.shared.accounts.account import Account
from norman_objects.shared.queries.query_constraints import QueryConstraints
from norman_objects.shared.security.sensitive import Sensitive
from pydantic import TypeAdapter

from norman_core.clients.http_client import HttpClient


class Accounts:
    @staticmethod
    async def get_accounts(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json = constraints.model_dump(mode="json")

        response = await http_client.post("persist/accounts/get", token, json=json)
        return TypeAdapter(dict[str, Account]).validate_python(response)

    @staticmethod
    async def create_accounts(http_client: HttpClient, token: Sensitive[str], accounts: list[Account]):
        json = TypeAdapter(list[Account]).dump_python(accounts, mode="json")

        response = await http_client.post("persist/accounts", token, json=json)
        return TypeAdapter(list[Account]).validate_python(response)

    @staticmethod
    async def replace_accounts(http_client: HttpClient, token: Sensitive[str], accounts: list[Account]):
        json = None
        if accounts is not None:
            json = TypeAdapter(list[Account]).dump_python(accounts, mode="json")

        modified_entity_count: int = await http_client.put("persist/accounts", token, json=json)
        return modified_entity_count

    @staticmethod
    async def update_accounts(http_client: HttpClient, token: Sensitive[str], account: Account.UpdateSchema, constraints: Optional[QueryConstraints] = None):
        parsed_constraints = None
        if constraints is not None:
            parsed_constraints = constraints.model_dump(mode="json")

        json = {
            "account": account.model_dump(mode="json"),
            "constraints": parsed_constraints
        }

        affected_entities_count: int = await http_client.patch("persist/accounts", token, json=json)
        return affected_entities_count

    @staticmethod
    async def delete_accounts(http_client: HttpClient, token: Sensitive[str], constraints: QueryConstraints):
        json = constraints.model_dump(mode="json")

        affected_entities_count: int = await http_client.delete("persist/accounts", token, json=json)
        return affected_entities_count



from norman_objects.services.file_pull.download.tracked_download_union import TrackedDownloadUnion
from norman_objects.services.file_pull.requests.asset_download_request import AssetDownloadRequest
from norman_objects.services.file_pull.requests.input_download_request import InputDownloadRequest
from norman_objects.services.file_pull.requests.output_download_request import OutputDownloadRequest
from norman_objects.shared.security.sensitive import Sensitive
from pydantic import TypeAdapter

from norman_core.clients.http_client import HttpClient


class FilePull:
    @staticmethod
    async def get_download_metadata(http_client: HttpClient, token: Sensitive[str], entity_id: str):
        response = await http_client.get(f"file-pull/metadata/{entity_id}", token)
        return TypeAdapter(TrackedDownloadUnion).validate_python(response)

    @staticmethod
    async def submit_asset_links(http_client: HttpClient, token: Sensitive[str], download_request: AssetDownloadRequest):
        json = download_request.model_dump(mode="json")

        response = await http_client.post("file-pull/upload/assets", token, json=json)
        return TypeAdapter(list[str]).validate_python(response)

    @staticmethod
    async def submit_input_links(http_client: HttpClient, token: Sensitive[str], download_request: InputDownloadRequest):
        json = download_request.model_dump(mode="json")

        response = await http_client.post("file-pull/upload/inputs", token, json=json)
        return TypeAdapter(list[str]).validate_python(response)

    @staticmethod
    async def submit_output_links(http_client: HttpClient, token: Sensitive[str], download_request: OutputDownloadRequest):
        json = download_request.model_dump(mode="json")

        response = await http_client.post("file-pull/upload/outputs", token, json=json)
        return TypeAdapter(list[str]).validate_python(response)



from io import BufferedReader
from typing import TypedDict, Any, Union, Iterator, AsyncIterator

from aiofiles.threadpool.binary import AsyncBufferedReader

FileStream = Union[BufferedReader, AsyncBufferedReader, bytes]

class RequestKwargs(TypedDict, total=False):
    content: Union[bytes, str]
    data: Union[dict[str, Any], bytes, str]
    files: dict[str, tuple[str, FileStream, str]]
    json: Any
    params: dict[str, Any]
    stream: Union[Iterator[bytes], AsyncIterator[bytes]]



from norman_objects.services.authenticate.login.account_id_password_login_request import AccountIDPasswordLoginRequest
from norman_objects.services.authenticate.login.api_key_login_request import ApiKeyLoginRequest
from norman_objects.services.authenticate.login.email_password_login_request import EmailPasswordLoginRequest
from norman_objects.services.authenticate.login.login_response import LoginResponse
from norman_objects.services.authenticate.login.name_password_login_request import NamePasswordLoginRequest

from norman_core.clients.http_client import HttpClient


class Login:
    @staticmethod
    async def login_default(http_client: HttpClient, account_id: str):
        response = await http_client.post(f"authenticate/login/default/{account_id}")
        return LoginResponse.model_validate(response)

    @staticmethod
    async def login_with_key(http_client: HttpClient, api_key_login_request: ApiKeyLoginRequest):
        response = await http_client.post("authenticate/login/key", json=api_key_login_request.model_dump(mode="json"))
        return LoginResponse.model_validate(response)

    @staticmethod
    async def login_password_account_id(http_client: HttpClient, login_request: AccountIDPasswordLoginRequest):
        response = await http_client.post("authenticate/login/password/account_id", json=login_request.model_dump(mode="json"))
        return LoginResponse.model_validate(response)

    @staticmethod
    async def login_password_name(http_client: HttpClient, login_request: NamePasswordLoginRequest):
        response = await http_client.post("authenticate/login/password/name", json=login_request.model_dump(mode="json"))
        return LoginResponse.model_validate(response)

    @staticmethod
    async def login_password_email(http_client: HttpClient, login_request: EmailPasswordLoginRequest):
        response = await http_client.post("authenticate/login/password/email", json=login_request.model_dump(mode="json"))
        return LoginResponse.model_validate(response)

    @staticmethod
    async def login_email_otp(http_client: HttpClient, email: str):
        await http_client.post("authenticate/login/email/otp", json={"email": email})

    @staticmethod
    async def verify_email_otp(http_client: HttpClient, email: str, code: str):
        response = await http_client.post("authenticate/login/email/otp/verify", json={"email": email, "code": code})
        return LoginResponse.model_validate(response)



from norman_core.services.authenticate.login import Login
from norman_core.services.authenticate.register import Register
from norman_core.services.authenticate.signup import Signup


class Authenticate:
    login = Login
    register = Register
    signup = Signup

__all__ = ['Authenticate']



from typing import Optional

from norman_objects.shared.notifications.notification import Notification
from norman_objects.shared.queries.query_constraints import QueryConstraints
from norman_objects.shared.security.sensitive import Sensitive
from pydantic import TypeAdapter

from norman_core.clients.http_client import HttpClient


class Notifications:
    @staticmethod
    async def get_notifications(http_client: HttpClient, token: Sensitive[str], constraints: Optional[QueryConstraints] = None):
        json = None
        if constraints is not None:
            json = constraints.model_dump(mode="json")

        response = await http_client.post("persist/notifications/get", token, json=json)
        return TypeAdapter(list[Notification]).validate_python(response)
